﻿using System;
using System.ServiceModel;
using System.Windows;
using System.IO.IsolatedStorage;
using System.Xml.Serialization;

namespace GestioIncidentalPhone
{

    public partial class MainPage
    {
        private Usuaris _usuari = new Usuaris();
        
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            OcultaMissatgeAvis();
            CarregaUltimAcces();
        }

        private void CarregaUltimAcces()
        {

            //Carrega les dades de l'usuari emmagatzemat (últim intent d'accés)
            using (IsolatedStorageFile userFile = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (userFile.FileExists("usuari.xml"))
                {
                    using (IsolatedStorageFileStream fileStream =
                      userFile.OpenFile("usuari.xml",
                                        System.IO.FileMode.Open))
                    {
                        XmlSerializer serialitzador = new XmlSerializer(typeof(Usuaris));
                        _usuari = (Usuaris)serialitzador.Deserialize(fileStream);

                        txtUsuari.Text = _usuari.Usuari;
                        //txtPassword.Password = _usuari.Password;
                    }
                }
            }

        }

        private void btnAccedir_Click(object sender, RoutedEventArgs e)
        {

            //Oculta si existeix algun missatge d'avís
            OcultaMissatgeAvis();

            MostraMissatge("Validant usuari, esperi si us plau...");

            if (txtUsuari.Text != "" && txtPassword.Password != "")
            {
                //Guardem l'usuari i la contrasenya introduïdes
                GuardarUsuari();
                //Comprovem l'accés de l'usuari
                ComprovarUsuari();
            }
            else
            {
                MostraMissatge("Ha d'introduir un usuari i una contrasenya");
            }
        }

        private void GuardarUsuari()
        {
            _usuari.Usuari = txtUsuari.Text;
            //_usuari.Password = txtPassword.Password;

            //Guardem l'usuari
            using (IsolatedStorageFile userFile = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (userFile.FileExists("usuari.xml")) userFile.DeleteFile("usuari.xml");

                using (IsolatedStorageFileStream fileStream =
                  userFile.OpenFile("usuari.xml", System.IO.FileMode.CreateNew))
                {
                    XmlSerializer serialitzador = new XmlSerializer(typeof(Usuaris));
                    serialitzador.Serialize(fileStream, _usuari);
                }
            }
        }

        private void ComprovarUsuari()
        {

            incidenciesServei.OServiceClient wcfClient = new incidenciesServei.OServiceClient();

            wcfClient.GetTecnicCompleted += ComprovaAccesTecnic;
            
            wcfClient.GetTecnicAsync(txtUsuari.Text, txtPassword.Password);

        }

        void ComprovaAccesTecnic(object sender, incidenciesServei.GetTecnicCompletedEventArgs tecnicAct)
        {
            try
            {
                if (tecnicAct.Cancelled == false || tecnicAct.Error == null)
                {
                    if (tecnicAct.Result.CodUsuari == 0)
                    {
                        //Si retorna 0, significa que no existeix un tècnic
                        //amb aquelles dades
                        MostraMissatge("Usuari o Password incorrecta");
                    }
                    else
                    {
                        OcultaMissatgeAvis();

                        //Es mostra la pantalla amb les incidències pendendents del tècnic identificat
                        //Passem el seu identificador y el seu nom complet a la pàgina d'incidències
                        string parametres = string.Format("codTecnic={0}&NomComplet={1}&Login={2}", tecnicAct.Result.CodUsuari, tecnicAct.Result.NomComplet, tecnicAct.Result.Login);
                        NavigationService.Navigate
                                   (new Uri("/PagInci.xaml?" + parametres, UriKind.Relative));

                    }

                }
            }

            catch (TimeoutException ex)
            {
                MostraMissatge("Temps excedit en la connexió." + ex.Message);
            }

            catch (CommunicationException ex)
            {
                MostraMissatge("Error en la comunicació." + ex.Message);
            }

        }

        //A l'obtenir focus si hi ha missatge d'error
        private void FocusEnUsuari(object sender, RoutedEventArgs e)
        {
            OcultaMissatgeAvis();
        }

        //A l'obtenir focus si hi ha missatge d'error
        private void FocusEnPassword(object sender, RoutedEventArgs e)
        {
            OcultaMissatgeAvis();
        }

        //Oculta missatge d'avís
        private void OcultaMissatgeAvis()
        {
            txbAvis.Text = "";
            txbAvis.Visibility = Visibility.Collapsed;
        }
        //Mostra missatge d'error o d'avís
        private void MostraMissatge(string missatge)
        {
            txbAvis.Text = missatge;
            txbAvis.Visibility = Visibility.Visible;
        }
    }
}
