﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Windows;
using Microsoft.Phone.Net.NetworkInformation;
using NetworkInterface = System.Net.NetworkInformation.NetworkInterface;

namespace GestioIncidentalPhone
{
    public partial class PagInci
    {
        //Estats Incidència
        private const string ESTAT_INC_TANCADA = "TANCADA";
        private const string ESTAT_INC_CANCEL = "CANCEL·LADA";

        //Llista d'incidències rebudes del servei web per a què el tècnic 
        //vegi les que en té pendents
        private IncRebuda _incSel = new IncRebuda();

        //Es crea l'objecte per a la connexió amb el servei
        private incidenciesServei.OServiceClient wcfClient = new incidenciesServei.OServiceClient();

        private int _idTec = 0;
        private string _login = "";
        private string _nomCompl = "";
        private string _estatActual = "";
        private Boolean _existConexServWeb;
    
        //S'obté el codUsuari del repartidor que s'ha validat i el seu nom complet
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {

            InitializeComponent();
            base.OnNavigatedTo(e);

            CarregaEvents();

            string loginbuit = "";
            string nombuit = "";

            //Obté l'identificador, login i nom complet del tècnic autenticat a la primera pantalla 
            if (NavigationContext.QueryString.TryGetValue("Login", out loginbuit))
            {
                _login = NavigationContext.QueryString["Login"];
                _idTec = int.Parse(NavigationContext.QueryString["codTecnic"]);
            }
            else
            {
                _login = "";
                _idTec = 0;
            }

            if (NavigationContext.QueryString.TryGetValue("NomComplet", out nombuit))
            {
                _nomCompl = NavigationContext.QueryString["NomComplet"];
            }
            else
            {
                _nomCompl = "";
            }

            //Mostrem el nom del tècnic validat a la pantalla
            txtTecnic.Text = _nomCompl;

            //Gravar en local les incidències del repartidor
            GravaBdIncRebudes();

    }

    //Carrega els events necessaris a la pantalla
    private void CarregaEvents()
    {
        //Event que enregistra un canvi a la xarxa del dispositiu
        DeviceNetworkInformation.NetworkAvailabilityChanged += CanviEnLaXarxa;

        //Event per a comprovar el servei web
        wcfClient.ExisteixConnexioServWebCompleted += ComprovarConnexioSerWeb;

        //Event per a actualitzar els nous estats de les incidències
        wcfClient.ActEstatIncCompleted += ActEstat;

        //Event per a carregar la llista d'incidències pendents, des del servidor central, per
        //al tècnic validat
        wcfClient.GetIncPendCompleted += CarregaLlistaIncPend;

    }

    //Grava en una BD SQL Server CE les incidències pendents que es rebin del servei web
    private void GravaBdIncRebudes()
    {
        wcfClient.GetIncPendAsync(_idTec);
    }

    //Carrega una llista amb les incidències rebudes del servei web
    void CarregaLlistaIncPend(object sender, incidenciesServei.GetIncPendCompletedEventArgs incPend)
    {

      try
      {

        if (incPend.Cancelled == false || incPend.Error == null)
        {


          //Base de dades amb les incidències pendents de transmetre al servei web 
          //quan el dispositiu tingui cobertura a la xarxa
            using (BDIncPendTrans contextPendTran = new BDIncPendTrans())
          {
            contextPendTran.CrearBd();
          }

          //Base de dades amb les incidències rebudes
            using (BDIncRebudes contextIncReb = new BDIncRebudes())
          {

            //Crea la BD per a carregar les incidències assignades al repartidor que haurà de tancar
            //o cancel·lar segons el cas
            contextIncReb.CrearBd();
            List<IncRebuda> listIncid = new List<IncRebuda>();

            incidenciesServei.Incidencia incRebud = new incidenciesServei.Incidencia();
            //Carreguem les incidències pendents en la base de dades BdIncRebudes
            //de la col·lecció obtinguda mitjançant el servei web, sempre i quan la incidència
            //no estigui pendent de transmetre's al servei web amb un canvi d'estat
            //(emmagatzemat a la base de dades BDIncPendTrans).
            for (int iRows = 0; iRows < incPend.Result.Count; iRows++)
            {
              incRebud = incPend.Result[iRows];
              if (!ExisteixIncEnBDPendTran(incRebud.CodIncidencia))
              {
                listIncid.Add(new IncRebuda() { CodIncidencia = incRebud.CodIncidencia, Sistema = incRebud.Sistema, Adreça = incRebud.Adreça });
              }
            }

            contextIncReb.IncidRebudes.InsertAllOnSubmit(listIncid);
            contextIncReb.SubmitChanges();
          }

          //Carrega a pantalla les incidències pendents
          MostraDades();
        }
      }
      catch (TimeoutException ex)
      {
        MessageBox.Show("Temps Excedit en la connexió." + ex.Message);
      }

      catch (CommunicationException ex)
      {
        MessageBox.Show("Error en la comunicació." + ex.Message);
      }

      catch (Exception ex)
      {
        MessageBox.Show("Excepció." + ex.Message);
      }

    }

    //Mostra en pantalla les incidències pendents que s'han rebut del servei web
    private void MostraDades()
    {
        using (BDIncRebudes context = new BDIncRebudes())
        {
            lstIncidencies.ItemsSource = context.IncidRebudes.ToList();
        }

    }

    //Intenta transmetre al servei web un canvi d'estat ('TANCADA')
    //la la incidència sel·leccionada a la pantalla.
    private void btnTancar_Click(object sender, RoutedEventArgs e)
    {

      //Actualitzem la incidència sel·leccionada del listbox como a estat ('TANCADA')
      _incSel = (IncRebuda)lstIncidencies.SelectedItem;

      if (_incSel == null)
      {
          MessageBox.Show("S'ha de sel·leccionar alguna incidència");
      }
      else
      {
          _estatActual = ESTAT_INC_TANCADA;
          ActualitzaEstatInc(_incSel.CodIncidencia, _estatActual);
      }
    }

    //Intenta transmetre al servei web un canvi d'estat ('CANCEL·LADA')
    //a la incidència sel·leccionada a la pantalla
    private void btnCancel_Click(object sender, RoutedEventArgs e)
    {
      //Actualitza la incidència sel·leccionada del listbox com a estat 'CANCEL·LADA'
        _incSel = (IncRebuda)lstIncidencies.SelectedItem;

        if (_incSel == null)
        {
            MessageBox.Show("S'ha de sel·leccionar alguna incidència");
        }
        else if (txtcancel.Text == "") 
        {
            MessageBox.Show("S'ha d'escriure algun motiu de la cancel·lació");
            txtcancel.Focus();
        }
        else
        {
            _estatActual = ESTAT_INC_CANCEL;
            ActualitzaEstatInc(_incSel.CodIncidencia, _estatActual, txtcancel.Text.Trim());
        }
    }

    //Actualitza l'estat de la incidència transmesa al servei web
    void ActEstat(object sender,
                    incidenciesServei.ActEstatIncCompletedEventArgs incActEstat)
    {

      try
      {

        if (incActEstat.Cancelled  || incActEstat.Error != null || incActEstat.Result == false)
        {
          //Si existeix algun error en la connexió s'emmagatzema la incidència tancada
          //en altra BD (Incidències pendents de transmetre) 
          //i quan torni a haver connexió s'enviaran les incidències d'aquesta BD al servei web
          GuardaIncPendTran(_estatActual);

        }

      }

      catch (TimeoutException ex)
      {
        MessageBox.Show("Temps Excedit a la connexió." + ex.Message);
      }

      catch (CommunicationException ex)
      {
        MessageBox.Show("Error en la comunicació." + ex.Message);
      }

      catch (Exception ex)
      {
        MessageBox.Show("Excepció." + ex.Message);
      }

      EsborrarIncRebuda();

      //Buidem el motiu de la cancel·lació
      txtcancel.Text = "";
    }
    private void ActualitzaEstatInc(int codInc, string nouEstat, string motiuCancel = "")
    {

      switch (nouEstat)
      {
        case ESTAT_INC_TANCADA:
          {
            wcfClient.ActEstatIncAsync(codInc, nouEstat, "");
            break;
          }

        case ESTAT_INC_CANCEL:
          {
              wcfClient.ActEstatIncAsync(codInc, nouEstat, motiuCancel);
            break;
          }

      }
    }

    //Esborra de la BD de les incidències rebudes la què s'ha tancat o cancel·lat
    private void EsborrarIncRebuda()
    {
        using (BDIncRebudes context = new BDIncRebudes())
      {
        context.GetTable<IncRebuda>().Attach(_incSel);
        context.GetTable<IncRebuda>().DeleteOnSubmit(_incSel);
        context.SubmitChanges();
        MostraDades();
      }

    }
    
    //Si s'obté un canvi a la xarxa i es tracta d'una connexió correcta
    //envia al servei web les incidències que es troben a la BD
    //incidències pendents de transmetre's.
    void CanviEnLaXarxa(object sender, NetworkNotificationEventArgs e)
    {
      if (e.NotificationType == NetworkNotificationType.InterfaceConnected)
      {
        TransmetreIncPendents(false);
      }
    }
    
    //Recorre la taula emmagatzemada en local de les incidències tancades o cancel·lades
    //que no es van poder transmetre pel servei web per falta de connexió
    //Aquest mètode torna a intentar transmetre aquestes dades.
    //Si el paràmetre bAmbComentaris és igual a 'true' mostrarà els missatges
    //al dispositiu.
    private void TransmetreIncPendents(Boolean bAmbcomentaris)
    {

      Boolean existienIncPendTran = false;

      ValidaConnexio();

      if (NetworkInterface.GetIsNetworkAvailable() & _existConexServWeb )
      {

          using (BDIncPendTrans contextPendTran = new BDIncPendTrans())
        {
          foreach (var incPendTrans in contextPendTran.IncidPendTrans)
          {
            existienIncPendTran = true;
            _estatActual = incPendTrans.NouEstat;

            ActualitzaEstatInc
              (incPendTrans.CodIncidencia, incPendTrans.NouEstat, incPendTrans.MotiuCancel);

            contextPendTran.GetTable<IncPendTrans>().DeleteOnSubmit(incPendTrans);
            contextPendTran.SubmitChanges();
          }
        }

        if (bAmbcomentaris)
        {
          if (!existienIncPendTran)
          {
            MessageBox.Show("No existeixen incidències pendents de transmetre's.");
          }
          else
          {
            MessageBox.Show("Incidències transmesses correctament al servidor central.");
          }
        }

      }

      else
      {
        if (bAmbcomentaris) MessageBox.Show("No existeix una connexió disponible");
      }

    }

    void ValidaConnexio()
    {

      //Comproba si existeix una connexió amb el servei web
      wcfClient.ExisteixConnexioServWebAsync();

    }

    //Comproba si hi ha connexió amb el servei Web
    void  ComprovarConnexioSerWeb(object sender,
                    incidenciesServei.ExisteixConnexioServWebCompletedEventArgs  conexServWeb)
    {
      try
      {
        if (conexServWeb.Cancelled || conexServWeb.Error != null )
        {
          _existConexServWeb = false;
        }
        else
        {
          _existConexServWeb = true;
        }

      }
        catch (Exception)
        {
          _existConexServWeb = false;
        }

    }

    //Pulsa sobre el botó Sincronitzar per a transmetre al servei web
    //els enviaments pendents de les incidències de la base de dades 
    //d'incidències pendents
    private void btnSincronitzar_Click(object sender, RoutedEventArgs e)
    {
      TransmetreIncPendents(true);
    }
    
    //Quan no existeix una connexió es guarden les incidències tancades o cancel·lades
    //en una base de dades temporal en local.
    private void GuardaIncPendTran(string nouEstat, string motiuCancel = "")
    {
        using (BDIncPendTrans contextPendTran = new BDIncPendTrans())
      {

          IncPendTrans novaincPendTrans = new IncPendTrans();

        novaincPendTrans.CodIncidencia = _incSel.CodIncidencia;
        novaincPendTrans.NouEstat = nouEstat;
        novaincPendTrans.MotiuCancel = motiuCancel;

        if (!ExisteixIncEnBDPendTran(_incSel.CodIncidencia))
        {
          //Si no existeix l'afegeixo
          contextPendTran.IncidPendTrans.InsertOnSubmit(novaincPendTrans);
          contextPendTran.SubmitChanges();
        }

      }
    }

    //Comprova si la incidència ja existeix a la base de dades temporal esperant a ser enviada
    private Boolean ExisteixIncEnBDPendTran(int codInc)
    {
        using (BDIncPendTrans contextPendTran = new BDIncPendTrans())
      {
          IncPendTrans existeixIncPenTrans = new IncPendTrans();
        existeixIncPenTrans = (from incPenTrans in contextPendTran.IncidPendTrans
                             where incPenTrans.CodIncidencia == codInc
                             select incPenTrans).FirstOrDefault();

        if (existeixIncPenTrans == null)
        {
          return false;
        }
        return true;
      }

    }
  }
}
