﻿Public Class FitxaUsuari

    Private ReadOnly _usuari As New Usuaris
    Private _carregant As Boolean
    Private _codUsuari As Integer
    Private _era_baixa As Boolean

    Public Sub Carregar(Optional ByVal codUsuari As Integer = 0)

        _codUsuari = codUsuari

        CarregarFinestra()
        Me.ShowDialog()

    End Sub

    Private Sub CarregarFinestra()

        If _codUsuari <> 0 Then
            'Modificació
            CarregaDades()
        Else
            'Nou
            BuidarDades()
        End If

    End Sub

    Private Sub BuidarDades()

        txtNom.Text = ""
        txtCognoms.Text = ""
        txtDNI.Text = ""
        dataAlta.SelectedDate = DateTime.Now.Date
        txtAdreça.Text = ""
        txtCP.Text = ""
        txtmobil.Text = ""
        txtTlf.Text = ""
        txtmail.Text = ""
        txtUsuari.Text = ""
        dataBaixa.DisplayDate = DATA_NULL
        dataBaixa.IsEnabled = False
        txtUsuariBaixa.IsEnabled = False
        chkBaixa.IsEnabled = False

    End Sub

    Private Sub CarregaDades()

        _carregant = True

        With _usuari
            .Buscar(_codUsuari)
            txtNom.Text = .Nom
            txtCognoms.Text = .Cognoms
            txtDNI.Text = .Dni
            dataAlta.SelectedDate = .Data_Alta

            txtAdreça.Text = .Adreça
            txtCP.Text = .Cp
            txtmobil.Text = .Mobil
            txtTlf.Text = .Telf
            txtmail.Text = .Email
            txtUsuari.Text = .Login
            txtUsuariBaixa.Text = .Usuari_Baixa
            dataBaixa.SelectedDate = .Data_Baixa
            chkBaixa.IsChecked = .EsBaixa()

            'CarregarPerfils
            rdAdministrador.IsChecked = .TePerfil(ADM)
            rdOficinista.IsChecked = .TePerfil(OFI)
            rdTecnic.IsChecked = .TePerfil(TEC)

        End With

        'Comprovem si al carregar les dades, l'usuari ja estava donat de baixa
        If chkBaixa.IsChecked Then
            _era_baixa = True
        Else
            _era_baixa = False
        End If

        _carregant = False

    End Sub

    Private Sub btnGuardar_Click(sender As Object, e As System.Windows.RoutedEventArgs) Handles btnGuardar.Click

        If ComprovarDades() Then

            With _usuari
                .codUsuari = _codUsuari
                .Nom = txtNom.Text.Trim
                .Cognoms = txtCognoms.Text.Trim
                .Dni = txtDNI.Text.Trim
                .Data_Alta = dataAlta.SelectedDate
                .Adreça = txtAdreça.Text.Trim
                .Cp = txtCP.Text.Trim
                .Mobil = txtmobil.Text.Trim
                .Telf = txtTlf.Text.Trim
                .Email = txtmail.Text.Trim
                .Login = txtUsuari.Text.Trim
                .Data_Baixa = IIf(dataBaixa.SelectedDate.ToString = "", DateTime.MinValue, dataBaixa.SelectedDate)
                .Usuari_Baixa = txtUsuariBaixa.Text.Trim

                Try
                    .Gravar()
                Catch ex As Exception
                    MessageBox.Show _
                    (ex.Message, "Error al guardar", MessageBoxButton.OK, MessageBoxImage.Error)
                End Try

            End With

            Close()

        End If

    End Sub

    Private Function ComprovarDades() As Boolean

        Try
            Comprovar(txtNom, "el nom")
            Comprovar(txtCognoms, "els cognoms")
            Comprovar(txtDNI, "el DNI")
            Comprovar(txtTlf, "el telèfon")
            Comprovar(txtUsuari, "l'usuari")

            ComprovarPerfils()

            'Comprovem si el DNI o el Login és igual a algun ja existent a la BD només si l'usuari
            'és nou (_codUsuari=0) o bé, l'usuari ja existeix, però aquestes dades han canviat
            With _usuari
                If _codUsuari <> 0 And .Dni <> txtDNI.Text.Trim Then
                    ComprovarDNIExistent()
                End If
                If _codUsuari <> 0 And .Login <> txtUsuari.Text.Trim Then
                    ComprovarUsuariExistent()
                End If

            End With

            ComprovarBaixa()

        Catch ex As Exception
            MessageBox.Show _
              (ex.Message, "Usuari", MessageBoxButton.OK, MessageBoxImage.Information)

            Return False
        End Try

        Return True

    End Function

    Private Sub ComprovarPerfils()

        If rdAdministrador.IsChecked = False And rdOficinista.IsChecked = False And rdTecnic.IsChecked = False Then
            Throw New Exception("S'ha de sel·leccionar un perfil")
        End If

        If Not IsNothing(_codUsuari) Then
            Dim incidencies As Incidencies = New Incidencies
            If incidencies.AssociadaIncidencia("codTecnic", _codUsuari) And _
                _usuari.TePerfil(TEC) And rdTecnic.IsChecked = False Then
                Throw New Exception("No es pot modificar el perfil de l'usuari perquè " & _
                                    "té associada alguna incidència oberta.")
            End If
        End If

    End Sub

    'Només la utilitzarem per mirar si es canvia el perfil d'un tècnic amb incidències assignades
    Private Function CanviPerfilOK() As Boolean

        Dim canvi As Boolean = False
        Dim incidencies As Incidencies = New Incidencies

        If _usuari.TePerfil(TEC) And incidencies.AssociadaIncidencia("codTecnic", _codUsuari) Then
            Throw New Exception("No es pot modificar el perfil de l'usuari perquè té associada alguna incidència")
        Else
            canvi = True
        End If

        Return canvi

    End Function

    Private Sub Comprovar(ByRef oControl As TextBox, _
                                  ByVal sNomDada As String)

        If oControl.Text = "" Then
            oControl.Focus()
            Throw New Exception("Ha d'introduir " & sNomDada & ".")
        End If
    End Sub

    Private Sub ComprovarDNIExistent()
        If _usuari.DadaExistent("dni", "'" & txtDNI.Text.Trim & "'") Then
            txtDNI.Focus()
            Throw New Exception("Ja existeix un usuari amb aquest DNI a la base de dades.")
        End If
    End Sub

    Private Sub ComprovarUsuariExistent()
        If _usuari.DadaExistent("login", "'" & txtUsuari.Text.Trim & "'") Then
            txtUsuari.Focus()
            Throw New Exception("Ja existeix un usuari amb aquest login a la base de dades.")
        End If
    End Sub

    Private Sub ComprovarBaixa()
        If chkBaixa.IsChecked Then
            If _era_baixa = False Then
                dataBaixa.SelectedDate = DateTime.Now.Date
                txtUsuariBaixa.Text = usuariAct.Login
            End If
        Else
            If _era_baixa Then
                dataBaixa.SelectedDate = DATA_NULL
                txtUsuariBaixa.Text = ""
            End If
        End If
    End Sub

    Private Sub rdAdministrador_Checked(sender As Object, e As System.Windows.RoutedEventArgs) Handles rdAdministrador.Checked

        If Not _carregant And CanviPerfilOK() Then
            _usuari.AfegirPerfil(ADM)
        End If

    End Sub

    Private Sub rdOficinista_Checked(sender As Object, e As System.Windows.RoutedEventArgs) Handles rdOficinista.Checked

        If Not _carregant And CanviPerfilOK() Then
            _usuari.AfegirPerfil(OFI)
        End If

    End Sub

    Private Sub rdTecnic_Checked(sender As Object, e As System.Windows.RoutedEventArgs) Handles rdTecnic.Checked

        If Not _carregant Then
            _usuari.AfegirPerfil(TEC)
        End If

    End Sub

    Private Sub txtCP_KeyDown(sender As Object, e As System.Windows.Input.KeyEventArgs) Handles txtCP.KeyDown

        NomesNumeros(sender, e)

    End Sub

    Private Sub btnSortir_Click(sender As Object, e As System.Windows.RoutedEventArgs) Handles btnSortir.Click

        Me.Close()

    End Sub

    Private Sub txtmobil_KeyDown(sender As Object, e As System.Windows.Input.KeyEventArgs) Handles txtmobil.KeyDown

        NomesNumeros(sender, e)

    End Sub

    Private Sub txtTlf_KeyDown(sender As Object, e As System.Windows.Input.KeyEventArgs) Handles txtTlf.KeyDown

        NomesNumeros(sender, e)

    End Sub

    Private Sub btnRestablirContrasenya_Click(sender As Object, e As System.Windows.RoutedEventArgs) Handles btnRestablirContrasenya.Click

        Dim iRes As Integer

        If txtUsuari.Text <> "" Then

            iRes = MessageBox.Show _
                           ("¿Està segur que desitja resetejar la contrasenya de l'usuari " & _
                            txtUsuari.Text & "?", "Canvi Contrasenya", _
                           MessageBoxButton.YesNo, MessageBoxImage.Question)

            If iRes = vbYes Then
                Try
                    _usuari.CanviarContrasenya(txtUsuari.Text.Trim)
                Catch ex As Exception
                    MessageBox.Show _
                    (ex.Message, "Error al resetejar la contrasenya", MessageBoxButton.OK, MessageBoxImage.Error)
                End Try
            End If
        End If
    End Sub

End Class
